<?php
/**
 * @file
 * Provides CCK integration for simplestar module
 */

/**
 * Implementation of hook_field_info().
 */
function simplestar_field_info() {
  return array(
    'simplestar' => array(
      'label' => t('Simplestar'),
      'description' => t('Simplestar rating widget.'),
    ),
  );
}

/**
 * Implementation of hook_field_settings().
 */
function simplestar_field_settings($op, $field) {
  switch ($op) {
    case 'database columns':
      return array(
        'simplestar' => array('type' => 'int', 'not null' => FALSE, 'sortable' => TRUE),
      );
  }
}

/**
 * Implementation of hook_content_is_empty().
 */
function simplestar_content_is_empty($item, $field) {
  if (empty($item['simplestar'])) {
    return TRUE;
  }
  
  return FALSE;
}

/**
 * Implementation of hook_widget_info().
 */
function simplestar_widget_info() {
  return array(
    'simplestar' => array(
      'label' => t('Simplestar'),
      'field types' => array('simplestar'),
      'multiple values' => CONTENT_HANDLE_CORE,
      'callbacks' => array(
        'default value' => CONTENT_CALLBACK_DEFAULT,
      ),
    )
  );
}

/**
 * Implementation of hook_widget_settings().
 */
function simplestar_widget_settings($op, $widget) {
  switch ($op) {
    case 'form':
      $form = array();
      $form['cancelshow'] = array(
        '#type' => 'checkbox',
        '#title' => t('Show cancel'),
        '#default_value' => $widget['cancelshow'],
        '#description' => t('Show the cancel option in the rating widget.'),
      );
      $form['onevoteonly'] = array(
        '#type' => 'checkbox',
        '#title' => t('One vote only'),
        '#default_value' => $widget['onevoteonly'],
        '#description' => t('Allow only one vote, after this rating widget will be disabled. Please note: you cannot use \'Show cancel\' and \'One vote only\' at the same time.'),
      );
      $form['split'] = array(
        '#type' => 'textfield',
        '#title' => t('Split'),
        '#default_value' => intval($widget['split']),
        '#description' => t('Number of splits for stars (Must be greater than or equal to zero).'),
        '#element_validate' => array('_simplestar_widget_settings_split_validate'),
      );
      $form['stars'] = array(
        '#type' => 'textfield',
        '#title' => t('Stars'),
        '#default_value' => intval($widget['stars'])>0 ? intval($widget['stars']) : 5,
        '#required' => FALSE,
        '#element_validate' => array('_simplestar_widget_settings_stars_validate'),
        '#description' => t('Number of stars (Must be greater than 1).'),
      );
      
      return $form;
    case 'save':
      return array('cancelshow', 'onevoteonly', 'split');
  }
}

/**
 * Validate split field value 
 */
function _simplestar_widget_settings_split_validate($element, &$form_state) {
  $value = $form_state['values']['split'];
  
  if ($value) {
    if (!is_numeric($value)) {
      form_set_error('min', t('"Split" must be a number.'));
    }
    elseif ($value < 0) {
      form_set_error('min', t('"Split" must be greater than or equal to zero.'));
    }
  }
}

/**
 * Validate the stars field value 
 */
function _simplestar_widget_settings_stars_validate($element, &$form_state) {
  $value = $form_state['values']['stars'];
  
  if ($value != '' && (!is_numeric($value)  || intval($value) < 2)) {
    form_set_error('stars', t('Please enter a valid number of stars value.'));
  }
}

/**
 * Implementation of hook_widget().
 */
function simplestar_widget(&$form, &$form_state, $field, $items) {
  $element = array(
    '#type' => 'simplestar',
    '#default_value' => isset($items[0]['simplestar']) ? $items[0]['simplestar'] : NULL,
    '#stars' => is_numeric($field['widget']['stars']) ? $field['widget']['stars'] : 5,
    '#onevoteonly' => $field['widget']['onevoteonly'],
    '#cancelshow' => $field['widget']['cancelshow'],
    '#split' => intval($field['widget']['split']),
  );
  
  return $element;
}

/**
 * Implementation of hook_field_formatter_info().
 */
function simplestar_field_formatter_info() {
  return array(
    'default' => array(
      'label' => t('Stars'),
      'field types' => array('simplestar'),
      'multiple values' => CONTENT_HANDLE_CORE,
    ),
    'rating' => array(
      'label' => t('Rating'),
      'field types' => array('simplestar'),
      'multiple values' => CONTENT_HANDLE_CORE,
    ),
    'percentage' => array(
      'label' => t('Percentage'),
      'field types' => array('simplestar'),
      'multiple values' => CONTENT_HANDLE_CORE,
    ),
  );
}

/**
 * Theme function for 'default' simplestar field formatter.
 */
function theme_simplestar_formatter_default($element) {
  if (empty($element['#item']['simplestar'])) {
    $element['#item']['simplestar'] = 0;
  }
  $simplestar = _simplestar_cck_field_settings($element);
  
  $settings = array(
    '#title' => $element['#title'],
    '#description' => $element['#description'],
    '#field_name' => $element['#field_name'],
    '#stars' => $simplestar['#post']['stars'],
    '#split' => intval($simplestar['#post']['split']),
  );

  return simplestar_static($element['#item']['simplestar'], $settings);
}

/**
 * Theme function for 'rating' simplestar field formatter.
 */
function theme_simplestar_formatter_rating($element) {
  if (!isset($element['#item']['simplestar'])) {
    $element['#item']['simplestar'] = 0;
  }

  return $element['#item']['simplestar'];
}

/**
 * Theme function for 'percentage' simplestar field formatter.
 */
function theme_simplestar_formatter_percentage($element) {
  if (!isset($element['#item']['simplestar'])) {
    $element['#item']['simplestar'] = 0;
  }
  $simplestar = _simplestar_cck_field_settings($element);
  
  $percentage = intval($simplestar['#post']['stars']) > 0 ? round($element['#item']['simplestar'] / intval($simplestar['#post']['stars']) * 100, 2) : 0;
  return $percentage;
}

/**
 * Get simplestar cck field settings
 */
function _simplestar_cck_field_settings($element) {
  module_load_include('inc', 'content', 'includes/content.crud');
  $item = content_field_instance_read(array('type_name' => $element['#type_name'], 'field_name' => $element['#field_name']), FALSE);
  $simplestar = $item[0]['widget']['default_value'][0];
  
  return $simplestar;
}